# -*- ecoding: utf-8 -*-
# @ModuleName: ocean_engine_creative
# @Function: 
# @Author: 安辉
# @Time: 2022/1/17 5:55 下午

import sys

sys.path.append('../')

import json
import traceback
from datetime import datetime
from utils.common import PERIOD_TYPE, LIST_TYPE, AGGR_CATEGORY_LIST, LANDING_TYPE, AGGR_APP_CODE, VIDEO_TYPE, \
    VIDEO_DURATION_TYPE, IMAGE_MODE, AGGR_BUSINESS_CODE, ORDER_BY
from utils.proxy_utils import fetch_proxy_by_service
from utils.redis_utils import RedisUtils
from utils.mongo_utils import MongoUtils
from utils.logs import OceanCreativeLogger
import requests
import smtplib
from email.mime.text import MIMEText
from email.header import Header
from email.mime.multipart import MIMEMultipart
import pandas as pd
import redis
from itertools import combinations, product
from tqdm import tqdm
from multiprocessing import Pool


class OceanCreative(object):
    def __init__(self):
        self.log = OceanCreativeLogger
        self.db_name = 'ocean_creative'
        self.db = 9
        self.redis_conn = RedisUtils(db=self.db)
        self.mg_collection = f'{self.db_name}_{datetime.now().strftime("%Y-%m-%d")}'
        self.mongo_conn = MongoUtils(db=self.db_name, collection=self.mg_collection)
        self.video_ids = []
        self.date = datetime.now().strftime("%Y{y}%m{m}%d{d}").format(y="年", m="月", d="日")
        self.url = 'https://cc.oceanengine.com/creative_radar_api/v1/material/list'
        self.headers = {
            'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.71 Safari/537.36',
            'referer': 'https://cc.oceanengine.com/inspiration/creative-radar/video',
            'accept-language': 'zh-CN,zh;q=0.9',
        }

    def send_email(self):
        # 发送邮箱服务器
        smtp_server = "smtp.qq.com"

        # 发送邮箱用户名密码
        user = "984007548@qq.com"
        password = "trpmnetjnvrhbedj"

        # 发送和接收邮箱
        sender = "984007548@qq.com"
        receive = "anhui@reyun.com"

        # 发送邮件主题和内容
        subject = "巨量创意素材标签"
        content = "<html><h1 style='color:red'>巨量创意素材标签</h1></html>"

        # HTML邮件正文
        # msg = MIMEText(content, 'html', 'utf-8')
        # msg['Subject'] = Header(subject, 'utf-8')
        # msg['From'] = "984007548@qq.com"
        # msg['To'] = "anhui@reyun.com"

        # SSL协议端口号要使用465
        smtp = smtplib.SMTP_SSL(smtp_server, 465)

        # HELO向服务器标志用户身份
        smtp.helo(smtp_server)

        # 服务器返回结果确认
        smtp.ehlo(smtp_server)

        # 登录邮箱服务器用户名密码
        smtp.login(user, password)

        send_file = open(r"./test.txt", "rb").read()

        att = MIMEText(send_file, "base64", 'utf-8')
        att['Content-Type'] = 'application/octet-stream'
        att['Content-Disposition'] = f'attachment;filename="{self.date}巨量创意素材标签.csv"'

        msgRoot = MIMEMultipart()
        msgRoot.attach(MIMEText(content, 'html', 'utf-8'))
        msgRoot['Subject'] = subject
        msgRoot['From'] = sender
        msgRoot['To'] = ','.join(receive)
        msgRoot.attach(att)

        self.log.info("Send email start...")
        smtp.sendmail(sender, receive, msgRoot.as_string())
        smtp.quit()
        self.log.info("email send end!")

    def test_params(self, list_type, aggr_category_list):
        params = (
            ('list_type', list_type),
            ('material_type', '3'),
            ('order_by', 'total_play'),
            ('period_type', '3'),
            ('aggr_app_code', '4'),
            ('aggr_category_list', aggr_category_list),
            ('video_type', '[]'),
            ('landing_type', '[]'),
            ('limit', '24'),
            ('page', '1'),
            ('video_duration_type', '5'),
        )
        return params

    def get_params(self, parameter, page):
        meta = {}
        params = (
            ('list_type', parameter[0]),
            ('material_type', '3'),
            ('order_by', 'total_play'),
            ('period_type', '3'),
            ('aggr_app_code', parameter[1]),
            ('aggr_category_list', parameter[2]),
            ('video_type', [parameter[3]]),
            ('landing_type', '[]'),
            # ('image_mode', parameter.get('image_mode')),
            ('video_duration_type', '5'),
            ('aggr_business_code', parameter[4]),
            ('limit', '24'),
            ('page', page),
        )
        meta['ad_type_id'] = parameter[0]
        meta['media_id'] = parameter[1]
        meta['video_type_id'] = parameter[3]
        return params, meta

    def parse(self, data):
        self.log.info('正在解析数据...')
        ad_data = data.get('data', {})
        materials = ad_data.get('materials', {})
        if not materials:
            self.log.info(f'data:{data}')
            self.log.info('当前参数组合暂无数据...')
            return []
        meta = data.get('meta', {})
        results = []
        for item in materials:
            result = {}

            # 广告类型
            ad_type_id = meta.get('ad_type_id', '')
            ad_type = LIST_TYPE.get(ad_type_id, '')

            # 素材ID
            material_id = item.get('material_id', '')

            # 行业
            first_industry = item.get('first_game_ad_industry_name', '')
            second_industry = item.get('second_game_ad_industry_name', '')
            industry = f'{first_industry}-{second_industry}'

            # 文案
            creative = item.get('bestTitle', '')
            # 素材类型
            video_type_id = meta.get('video_type_id')
            material_type = VIDEO_TYPE.get(video_type_id, [])

            # 推广目标
            landing_type = item.get('landing_type', '')
            promotion_target = LANDING_TYPE.get(landing_type, '')

            media_id = meta.get('media_id', '')
            media = AGGR_APP_CODE.get(media_id, '')

            video_id = item.get('vid', '')
            material_url = f'https://api.amemv.com/aweme/v1/play/?video_id={video_id}'

            result['material_id'] = material_id
            result['creative'] = creative
            result['material_type'] = material_type
            result['ad_type'] = ad_type
            result['industry'] = industry
            result['promotion_target'] = promotion_target
            result['media'] = media
            result['material_url'] = material_url
            # if video_id in self.video_ids:
            #     self.log.info('数据重复,已过滤...')
            #     continue
            # if video_id not in self.video_ids:
            #     self.video_ids.append(video_id)
            #     self.mongo_conn.insert(result)
            #     self.log.info(f'新增{len(self.video_ids)}条数据')
            results.append(result)
        self.mongo_conn.insert(results)
        self.log.info('数据解析完毕...')

    def get_response(self, params, meta=None):
        try:
            _, _, _, proxies = fetch_proxy_by_service()
            proxies = proxies if sys.platform not in ['win32', 'darwin'] else None
            response = requests.get(url=self.url, headers=self.headers, params=params, proxies=proxies, verify=False,
                                    timeout=5)
            data = response.json()
            data['meta'] = meta
            if not meta:
                return data
        except:
            traceback.print_exc()
        else:
            self.parse(data)

    def run(self, parameter):
        page = 1
        self.log.info(f'当前参数组合:{parameter}')
        params, meta = self.get_params(parameter, page)
        self.get_response(params, meta)


if __name__ == '__main__':
    industry = AGGR_CATEGORY_LIST.values()
    industry_ids = [i.get('child_ids') for i in industry]
    parameters = product(LIST_TYPE.keys(), [4, 8], industry_ids, VIDEO_TYPE.keys(), AGGR_BUSINESS_CODE.keys())
    parameters = list(parameters)
    parameter = parameters.pop()
    oc = OceanCreative()
    while True:
        lens = len(parameters)
        if lens <= 0:
            break
        batch = 10 if lens >= 10 else lens
        pool = Pool(batch)
        pool.apply_async(oc.run, (parameter,))
        pool.close()
        pool.join()
