# -*- ecoding: utf-8 -*-
# @ModuleName: logs
# @Function: 
# @Author: 安辉
# @Time: 2022/1/19 3:01 下午

import re
import logging
import sys
from logging.handlers import TimedRotatingFileHandler

if sys.platform in ["win32", "darwin"]:
    PRJ_DEBUG = True
else:
    PRJ_DEBUG = False


class Log(object):
    """日志类"""

    def __init__(self, name, filename, level=logging.INFO):
        self.logger = logging.getLogger(name)
        # 控制日志文件中记录级别

        self.formatter = logging.Formatter('%(levelname)s|%(asctime)s|%(module)s|%(funcName)s|%(message)s')

        # 生产环境
        if not PRJ_DEBUG:
            # 日志保留3天,一天保存一个文件
            self.fh = TimedRotatingFileHandler(filename, when='midnight', interval=1, backupCount=3)
            # 删除设置
            self.fh.suffix = '%Y-%m-%d.log'
            self.fh.extMatch = re.compile(r'^\d{4}-\d{2}-\d{2}.log$')
            # 定义日志文件中格式
            self.fh.setFormatter(self.formatter)
            self.logger.addHandler(self.fh)
            self.logger.setLevel(level)
        else:
            # 控制输出到控制台日志格式、级别
            self.ch = logging.StreamHandler()
            self.ch.setFormatter(self.formatter)
            self.logger.addHandler(self.ch)
            self.logger.setLevel(logging.DEBUG)

    def debug(self, msg):
        self.logger.debug(msg)

    def warning(self, msg):
        self.logger.warning(msg)

    def info(self, msg):
        self.logger.info(msg)

    def error(self, msg):
        self.logger.error(msg)

    def critical(self, msg):
        self.logger.critical(msg)

    def exception(self, msg):
        self.logger.exception(msg)

    def close(self):
        self.logger.removeHandler(self.fh)


OceanCreativeLogger = Log('ocean_creative', '/data/logs/ocean_engine/ocean_creative.log')
